#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "gequel.h"

int getSortKey(Params *p)
{
   int y;
   
   ldebug(2,"getSortKey called\n");
   
   /* Find unique primary key */
   for(y=0;y<p->numflds;y++)
   {
      if(IS_PRI_KEY(p->fields[y].flags)&&IS_UNIQUE(p->fields[y].flags))
      {
	 writeLog("Using unique primary key %d for sort in %s\n",y, p->table);
	 return y;
      }
   }
   
   /* Else, find primary key */
   for(y=0;y<p->numflds;y++)
   {
      if(IS_PRI_KEY(p->fields[y].flags))
      {
	 writeLog("Using primary key %d for sort in %s\n",y, p->table);
	 return y;
      }
   }
   
   /* Else, find first unique key */
   for(y=0;y<p->numflds;y++)
   {
      if(IS_UNIQUE(p->fields[y].flags))
      {
	 writeLog("Using unique key %d for sort in %s\n",y, p->table);
	 return y;
      }
   }
   
   writeLog("No unique and/or primary key in %s,"
	    "using first field (0).\n", p->table);
   return 0;
}

/* If we come here, then the user has selected a row in the list. */
void field_selected( GtkWidget      *clist,
		      gint            row,
		      gint            column,
		      GdkEventButton *event,
		      gpointer        data )
{
   TabInfo *ti = (TabInfo *)data;

   ldebug(2,"field_selected entered\n");

   /* This only catches <SPACE> or a doubleclick on the selected record !! : */
   if (event == NULL || event->type == GDK_2BUTTON_PRESS)
   {   
      ldebug(2,"field_selected: doubleclick detected\n");
      
      if(ti->table == NULL)
      {
	 showMessage("Currently no table selected !\n");
	 return;
      }
      writeLog("Setting sort field to %d (%s)\n",
	       row,ti->fields[row].name);
      /* watch out: next destroys current ti struct and forces a refresh !! */
      setSortKey(row);
      return;
   }
   else
      ldebug(3,"You selected field %d: no effect (use doubleclick !)\n",row);
}

gchar *getFieldTypeString(Params *p, int index)
{
   ldebug(2,"getFieldTypeString entered : fieldtype %d\n",p->fields[index].type);
   
   /* Next uses fields deprecated since MySQL 4.0 but still necessary for correct results */
   
   if(p->fields[index].flags & ENUM_FLAG ) return "Enum";
   if(p->fields[index].flags & BLOB_FLAG ) return "Blob";
   if(p->fields[index].flags & SET_FLAG  ) return "Set";
   if(p->fields[index].flags & TIMESTAMP_FLAG ) return "Timestamp";
   
   switch(p->fields[index].type)
   {
    case FIELD_TYPE_STRING:     return "String";    break;
    case FIELD_TYPE_VAR_STRING: return "VarString"; break;
    case FIELD_TYPE_FLOAT:      return "Float";     break;
    case FIELD_TYPE_DOUBLE:     return "Double";    break;
    case FIELD_TYPE_DECIMAL:    return "Decimal";   break;
    case FIELD_TYPE_LONGLONG:
    case FIELD_TYPE_INT24:
    case FIELD_TYPE_TINY:
    case FIELD_TYPE_SHORT:
    case FIELD_TYPE_LONG:       return "Integer";   break;
    case FIELD_TYPE_DATE:       return "Date";      break;
    case FIELD_TYPE_TIME:       return "Time";      break;
    case FIELD_TYPE_TIMESTAMP:  return "TimeStamp"; break;
    case FIELD_TYPE_DATETIME:   return "Datetime";  break;
    case FIELD_TYPE_YEAR:       return "Year";      break;
    case FIELD_TYPE_SET:        return "Set";       break;
    case FIELD_TYPE_ENUM:       return "Enum";      break;
    case FIELD_TYPE_TINY_BLOB:
    case FIELD_TYPE_MEDIUM_BLOB:
    case FIELD_TYPE_LONG_BLOB:
    case FIELD_TYPE_BLOB:       return "Blob:";     break;
    default:
      writeLog("Unknown fieldtype %d detected !\n",
	       p->fields[index].type);
      return "Unknown"; break;
   }
}

#define FIELDCOLS 8 /* Number of columns (this is here not the same as nrflds !!) */
void showFields(Params *p)
{
   gchar lengthstring[10];
   int   x;
   TabInfo *ti;
   gchar decimalstring[10];
   gchar *row[FIELDCOLS];
   gchar *titles[FIELDCOLS] =
   { "Fieldname", "Tablename", "Length", "Type", "Default", "Decimals", "Unique", "Not Null" };
   
   ldebug(2,"showFields entered\n");
   if(p->table == NULL)
   {
      showMessage("Cannot show fields: select a table first !\n");
      return;
   }
   
   destroyTab(p->tabs[FIELDSTAB]);
   /* destroy possible previous FieldTab + TabInfo */
   
   ti = newTabInfostruct(p, FIELDSTAB, FIELDCOLS, p->numflds);
   
   for(x=0;x<FIELDCOLS;x++)
     ti->titles[x] = titles[x];
   
   newTab(ti,field_selected,"fieldsclist");
   setTabColumnWidths(ti,FIELDSTAB);
   
   gtk_clist_freeze(GTK_CLIST(ti->clist));

   for(x=0; x < p->numflds; x++)
   {
      /* convert field length and nr. of decimals to string: */
      snprintf(lengthstring,sizeof(lengthstring) - 1,"%8lu",p->fields[x].length);
      snprintf(decimalstring,sizeof(decimalstring) - 1,"%8d",p->fields[x].decimals);
      
      row[0] = p->fields[x].name;
      row[1] = p->table;
      row[2] = lengthstring;
      row[3] = getFieldTypeString(p, x);
      row[4] = p->fields[x].def;
      row[5] = decimalstring;
      row[6] = IS_UNIQUE(p->fields[x].flags) ? "Yes" : "No";
      row[7] = IS_NOT_NULL(ti->fields[x].flags) ? "Yes" : "No";

      gtk_clist_append(GTK_CLIST(ti->clist),row);
      /* change the sortkey color : */
      if(x == ti->sortkey)
	gtk_clist_set_row_style(GTK_CLIST(ti->clist),
				x, ti->styles[STLRED]);
   }
   
   gtk_clist_thaw(GTK_CLIST(ti->clist));
   gtk_notebook_set_page (GTK_NOTEBOOK(p->notebook), FIELDSTAB);

   p->tabs[FIELDSTAB] = ti;
}

