#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "gequel.h"

/* If we come here, then the user has selected a row in the list. */
void qrecord_selected( GtkWidget      *clist,
		      gint            row,
		      gint            column,
		      GdkEventButton *event,
		      gpointer        data )
{
/*   TabInfo *ti = (TabInfo *)data; */
   
   ldebug(2,"qrecord_selected entered\n");
   
   /* This only catches <SPACE> or a doubleclick on the selected record !! : */
   if (event == NULL || event->type == GDK_2BUTTON_PRESS)
   {   
      ldebug(2,"qrecord_selected: doubleclick detected\n");
      showMessage("Sorry: can't open free query records\n"
		  "in a separate window !");
   }
}

/* TODO : combine with showRecords ? */
void showQuery(Params *p, char *query)
{
   int     x;
   SQL_ROW row;
   SQL_RES *res;
   TabInfo *ti;
   
   ldebug(2,"showQuery entered\n");
   if(p->table == NULL)
   {
      showMessage("Cannot show select query:\n select a table first !");
      return;
   }
   
   if(strncasecmp(query,"select ",7) != 0)
   {
      showMessage("Cannot execute query: Currently only\n"
		  "SELECT statements allowed. Sorry !");
      return;
   }
   
   if (executeQuery(p->dbsock, query))
   {
      showSqlError(p->dbsock,"Query in showQuery failed !");
      return;
   }
      
   destroyTab(p->tabs[QUERYTAB]);
   /* destroy possible previous QueryTab + TabInfo */

   res = sql_store_result(p->dbsock);
   ti  = newTabInfostruct(p, QUERYTAB, 
			     sql_num_fields(res), sql_num_rows(res));
   
   /* Now fetch the fieldnames for this selection : */
   for(x=0;x<ti->nrcols;x++)
     ti->titles[x] = res->fields[x].name;
   
   newTab(ti,qrecord_selected, "queryclist");
   setTabColumnWidths(ti,QUERYTAB);
   
   gtk_clist_freeze(GTK_CLIST(ti->clist));
   
   for(x=0;x < ti->nrrows;x++)
   {
      if((row = sql_fetch_row(res)) == NULL)
      {
	 showSqlError(p->dbsock,"Error reading record %d !",x);
	 break;
      }
      
      /* write a row to the browser : */
      gtk_clist_append(GTK_CLIST(ti->clist),row);
   }
   
   gtk_clist_thaw(GTK_CLIST(ti->clist));
   gtk_notebook_set_page (GTK_NOTEBOOK(p->notebook), QUERYTAB);

   p->tabs[QUERYTAB] = ti;
}

void loadQueryAction(GtkWidget *w, gpointer user_data)
{
   FILE *input;
   char qline[MAXQUERY];
   Params *p = (Params *) user_data;
   GtkTextIter    txtstart,txtend;
   
   ldebug(2,"loadQueryAction entered\n");
   
   input = fopen(p->selectedname,"r");
   if(input == NULL)
   {
      showVarArgMessage("Cannot load query:\n open file %s failed !",
			p->selectedname);
      return;
   }
   /* clear current buffer : */
   gtk_text_buffer_get_start_iter (p->querybuffer, &txtstart);
   gtk_text_buffer_get_end_iter (p->querybuffer, &txtend);
   gtk_text_buffer_delete(p->querybuffer, &txtstart, &txtend);

   gtk_text_buffer_get_start_iter (p->querybuffer, &txtstart);
   while(fgets(qline,MAXQUERY-1,input) != NULL)
     gtk_text_buffer_insert (p->querybuffer, &txtstart, qline, -1);

   fclose(input);
}

void saveQueryAction(GtkWidget *w, gpointer user_data)
{
   FILE *output;
   Params *p = (Params *) user_data;
   GtkTextIter    txtstart;
   GtkTextIter    txtend;
   
   ldebug(2,"saveQueryAction entered\n");
   
   output = fopen(p->selectedname,"w");
   if(output == NULL)
   {
      showVarArgMessage("Cannot save query:\n open file %s failed !",
		  p->selectedname);
      return;
   }
   gtk_text_buffer_get_start_iter (p->querybuffer, &txtstart);
   gtk_text_buffer_get_end_iter   (p->querybuffer, &txtend);
   fputs(gtk_text_buffer_get_text(p->querybuffer,&txtstart,&txtend,FALSE),output);
   
   fclose(output);
}

