#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "gequel.h"

/* FIX ME: use record_selected instead or do we need more checks ? */
/* If we come here, then the user has selected a row in the list. */
void srecord_selected( GtkWidget      *clist,
		       gint            row,
		       gint            column,
		       GdkEventButton *event,
		       gpointer        data )
{
   TabInfo *ti = (TabInfo *)data;
   
   ldebug(2,"srecord_selected entered\n");
   
   ti->selectedrow = row; /* single click : just preserve selection */
   ldebug(3,"You selected row %d.\n ",row);
   
   /* This only catches <SPACE> or a doubleclick on the selected record !! : */
   if (event == NULL || event->type == GDK_2BUTTON_PRESS)
   {   
      ldebug(2,"srecord_selected: doubleclick detected\n");
      
      if(ti->table == NULL)
      {
	 showMessage("Currently no table selected !");
	 return;
      }
      
      ldebug(3,"You doubleclicked row %d. More specifically you clicked in "
	     "column %d\n\n", row, column);
      
      newModifyWindow(ti);
   }
}

char *buildSearchQuery(Record *rec)
{
   char *query;
   int x,y;
   int size = 100; /* safe start */
   
   ldebug(2,"buildSearchQuery entered\n");
   
   /* get approximate querylength: account for quotes,spaces and equalsigns */
   for(y=0;y<rec->numflds;y++)
     size += strlen(rec->fields[y].name) + rec->fields[y].length + 10;
   query =(char *)g_malloc(size * sizeof(char));
   
   snprintf(query, size - 1, SELECT WHERE, rec->table);
   
   x=0;
   for(y=0;y<rec->numflds;y++)
   {
      gchar *entry = (gchar *)gtk_entry_get_text(rec->entries[y]);
      
      if(strlen(entry) > 0)
      {	 
	 if(x++ > 0) strcat(query, AND);
	 strcat (query, rec->fields[y].name);
	 strcat (query, LIKE);
	 strcat (query, addStringQuotes(rec->fields[y].type));
	 strncat(query, entry, rec->fields[y].length);
	 strcat (query, addStringQuotes(rec->fields[y].type));
      }
   }
   strcat(query,ORDERBY);
   strcat(query,rec->fields[rec->sortkey].name);
   
   return query;
   /* don't forget to free query after use !! */
}

/* TODO : combine with showRecords ? */
void showSearch(Params *p, Record *rec)
{
   int     x;
   SQL_ROW row;
   SQL_RES *res;
   TabInfo *ti;
   char *query;
   
   ldebug(2,"showSearch entered\n");
   if(p->table == NULL)
   {
      showMessage("Cannot show search matches:\n select a table first !");
      return;
   }
   
   query = buildSearchQuery(rec);
   
   if (executeQuery(p->dbsock, query))
   {
      showSqlError(p->dbsock,"Query in showSearch failed !");
      return;
   }
   free(query); /* buildSearchQuery mallocs, but relies on external free */
   
   destroyTab(p->tabs[SEARCHTAB]);
   /* destroy possible previous SearchTab + TabInfo */

   res = sql_store_result(p->dbsock);
   ti  = newTabInfostruct(p, SEARCHTAB, 
			  sql_num_fields(res), sql_num_rows(res));
   
   /* Now fetch the fieldnames for this selection : */
   for(x=0;x<ti->numflds;x++)
     ti->titles[x] = ti->fields[x].name;
   
   newTab(ti, srecord_selected, "searchclist");
   setTabColumnWidths(ti,SEARCHTAB);
   
   gtk_clist_freeze(GTK_CLIST(ti->clist));
   
   for(x=0;x < ti->nrrows;x++)
   {
      if((row = sql_fetch_row(res)) == NULL)
      {
	 showSqlError(p->dbsock,"Error reading record %d !",x);
	 break;
      }
      
      /* write a row to the browser : */
      gtk_clist_append(GTK_CLIST(ti->clist),row);
   }
   
   gtk_clist_thaw(GTK_CLIST(ti->clist));
   gtk_notebook_set_page (GTK_NOTEBOOK(p->notebook), SEARCHTAB);

   p->tabs[SEARCHTAB] = ti;
}
